@@ -10,8 +10,11 @@
 """
 XPath 3.1 implementation
 """
+from typing import ClassVar, Dict, Tuple
+
 from ..namespaces import XPATH_MAP_FUNCTIONS_NAMESPACE, \
     XPATH_ARRAY_FUNCTIONS_NAMESPACE  # , XSLT_XQUERY_SERIALIZATION_NAMESPACE
+from ..datatypes import QName
 from ..xpath30 import XPath30Parser
 
 
@@ -21,27 +24,7 @@ class XPath31Parser(XPath30Parser):
     """
     version = '3.1'
 
-    SYMBOLS = XPath30Parser.SYMBOLS | {
-        # Map and array functions
-        'map', 'array',  # 'merge',
-        'size', 'keys', 'contains', 'get',  # 'find',
-        'put',  # 'entry',
-        # 'remove', 'append', 'subarray', 'remove', 'join', 'flatten',
-
-        # 'random-number-generator', 'collation-key',
-        # 'contains-token', 'parse-ietf-date',
-
-        # Higher-order functions
-        # 'sort', 'apply', 'load-xquery-module', 'transform',
-
-        # Functions on JSON Data
-        # 'parse-json', 'json-doc', 'json-to-xml', 'xml-to-json',
-
-        # Arrow operator
-        # '=>',
-    }
-
-    DEFAULT_NAMESPACES = {
+    DEFAULT_NAMESPACES: ClassVar[Dict[str, str]] = {
         'map': XPATH_MAP_FUNCTIONS_NAMESPACE,
         'array': XPATH_ARRAY_FUNCTIONS_NAMESPACE,
         **XPath30Parser.DEFAULT_NAMESPACES
@@ -54,4 +37,4 @@ class XPath31Parser(XPath30Parser):
         'schema-attribute', 'schema-element', 'switch', 'text', 'typeswitch',
     }
 
-    function_signatures = XPath30Parser.function_signatures.copy()
+    function_signatures: Dict[Tuple[QName, int], str] = XPath30Parser.function_signatures.copy()
